﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Globalization;
using Windows.Storage.BulkAccess;
using Windows.Storage.FileProperties;
using System.Threading.Tasks;

namespace PhotoViewer.Exif
{
   public struct ExifMetadata
   {
      private string cameraModel;
      private ushort? colorRepresentation;
      private string creationSoftware;
      private DateTimeOffset dateTaken;
      private string equipmentManufacturer;
      private decimal? exposureCompensation;
      private ushort? exposureMode;
      private decimal? exposureTime;
      private ushort? flashMode;
      private decimal? focalLength;
      private uint? height;
      private decimal? horizontalResolution;
      private ushort? isoSpeed;
      private decimal? lensAperature;
      private ushort? lightSource;
      private decimal? verticalResolution;
      private uint? width;
      private string path;

      public static async Task<ExifMetadata> Create(FileInformation fileInformation)
      {
         IDictionary<string, object> metadata = await GetImageProperties(fileInformation);
         var exif = new ExifMetadata();

         exif.path = fileInformation.Path;

         // http://msdn.microsoft.com/en-us/library/ee872003(VS.85).aspx
         exif.creationSoftware = QueryMetadata<string>(metadata, "System.ApplicationName");
         exif.dateTaken = QueryMetadata<DateTimeOffset>(metadata, "System.Photo.DateTaken");
         exif.cameraModel = QueryMetadata<string>(metadata, "System.Photo.CameraModel");
         exif.equipmentManufacturer = QueryMetadata<string>(metadata, "System.Photo.CameraManufacturer");
         exif.exposureCompensation = QueryMetadata<decimal>(metadata, "System.Photo.ExposureBias");
         exif.exposureMode = QueryMetadata<ushort>(metadata, "System.Photo.ProgramMode");
         exif.exposureTime = QueryMetadata<decimal>(metadata, "System.Photo.ExposureTime");
         exif.flashMode = QueryMetadata<ushort>(metadata, "System.Photo.Flash");
         exif.focalLength = QueryMetadata<decimal>(metadata, "System.Photo.FocalLength");
         exif.isoSpeed = QueryMetadata<ushort>(metadata, "System.Photo.ISOSpeed");
         exif.lensAperature = QueryMetadata<decimal>(metadata, "System.Photo.Aperture");
         exif.lightSource = QueryMetadata<ushort>(metadata, "System.Photo.LightSource");
         exif.colorRepresentation = QueryMetadata<ushort>(metadata, "System.Image.ColorSpace");
         exif.verticalResolution = QueryMetadata<decimal>(metadata, "System.Image.VerticalResolution");
         exif.horizontalResolution = QueryMetadata<decimal>(metadata, "System.Image.HorizontalResolution");
         exif.width = fileInformation.ImageProperties.Width;
         exif.height = fileInformation.ImageProperties.Height;

         return exif;
      }

      public string Path
      {
         get
         {
            return this.path;
         }
      }

      public string CameraModel
      {
         get
         {
            return this.cameraModel;
         }
      }

      public ColorRepresentation ColorRepresentation
      {
         get
         {
            ColorRepresentation value = Exif.ColorRepresentation.Uncalibrated;
            if (this.colorRepresentation.HasValue)
            {
               if (!String.IsNullOrWhiteSpace(Enum.GetName(typeof(ColorRepresentation), this.colorRepresentation)))
               {
                  value = (ColorRepresentation)this.colorRepresentation;
               }
            }

            return value;
         }
      }

      public string CreationSoftware
      {
         get
         {
            return this.creationSoftware;
         }
      }

      public DateTimeOffset DateTaken
      {
         get
         {
            return this.dateTaken;
         }
      }

      public string EquipmentManufacturer
      {
         get
         {
            return this.equipmentManufacturer;
         }
      }

      public decimal? ExposureCompensation
      {
         get
         {
            return this.exposureCompensation;
         }
      }

      public ExposureMode ExposureMode
      {
         get
         {
            ExposureMode value = Exif.ExposureMode.Unknown;
            if (this.exposureMode.HasValue)
            {
               if (!String.IsNullOrWhiteSpace(Enum.GetName(typeof(ExposureMode), this.exposureMode)))
               {
                  value = (ExposureMode)this.exposureMode;
               }
            }

            return value;
         }
      }

      public decimal? ExposureTime
      {
         get
         {
            return this.exposureTime;
         }
      }

      public FlashMode FlashMode
      {
         get
         {
            FlashMode value = FlashMode.FlashDidNotFire;
            if (this.flashMode.HasValue && this.flashMode % 2 == 1)
            {
               value = FlashMode.FlashFired;
            }

            return value;
         }
      }

      public decimal? FocalLength
      {
         get
         {
            return this.focalLength;
         }
      }

      public uint? Height
      {
         get
         {
            return this.height;
         }
      }

      public decimal? HorizontalResolution
      {
         get
         {
            return this.horizontalResolution;
         }
      }

      public ushort? IsoSpeed
      {
         get
         {
            return this.isoSpeed;
         }
      }

      public decimal? LensAperture
      {
         get
         {
            return this.lensAperature;
         }
      }

      public LightSource LightSource
      {
         get
         {
            LightSource value = Exif.LightSource.Unknown;
            if (this.lightSource.HasValue)
            {
               if (!String.IsNullOrWhiteSpace(Enum.GetName(typeof(LightSource), this.lightSource)))
               {
                  value = (LightSource)this.lightSource;
               }
            }

            return value;
         }
      }

      public decimal? VerticalResolution
      {
         get
         {
            return this.verticalResolution;
         }
      }

      public uint? Width
      {
         get
         {
            return this.width;
         }
      }

      private static async Task<IDictionary<string, object>> GetImageProperties(FileInformation fileInformation)
      {
         string[] requests = 
           {
              "System.ApplicationName",
              "System.Photo.CameraManufacturer",
              "System.Photo.CameraModel",
              "System.Photo.DateTaken",
              "System.Photo.ExposureBias",
              "System.Photo.ProgramMode",
              "System.Photo.ExposureTime",
              "System.Photo.Flash",
              "System.Photo.FocalLength",
              "System.Photo.ISOSpeed",
              "System.Photo.Aperture",
              "System.Photo.LightSource",
              "System.Image.ColorSpace",
              "System.Image.VerticalResolution",
              "System.Image.HorizontalResolution"
           };

         return await fileInformation.ImageProperties.RetrievePropertiesAsync(requests);
      }

      private static T QueryMetadata<T>(IDictionary<string, object> metadata, string key)
      {
         T result;
         if (metadata.ContainsKey(key))
         {
            try
            {
               object queryResult = metadata[key];
               if (queryResult.GetType() == typeof(T))
               {
                  result = (T)queryResult;
               }
               else
               {
                  try
                  {
                     result = (T)Convert.ChangeType(queryResult, typeof(T));
                  }
                  catch (InvalidCastException)
                  {
                     result = default(T);
                  }
                  catch (FormatException)
                  {
                     result = default(T);
                  }
                  catch (OverflowException)
                  {
                     result = default(T);
                  }
               }
            }
            catch
            {
               result = default(T);
            }
         }
         else
         {
            result = default(T);
         }

         return result;
      }
   }
}